# coding: utf-8
from __future__ import absolute_import, unicode_literals

TYPE = "typ"
ALGORITHM = "alg"
KEY_ID = "kid"

ISSUER = "iss"
SUBJECT = "sub"
AUDIENCE = "aud"
EXPIRATION_TIME = "exp"
NOT_BEFORE = "nbf"
ISSUED_AT = "iat"
JWT_ID = "jti"

TOKEN = "token"
SESSION_ID = "sid"
USER_ID = "uid"
EMAIL = "email"
SCOPES = "scopes"

TOKEN_SESSION = "session"
TOKEN_AUTHORIZATION = "auth"
